#pragma once

#include <memory>
#include <string>
#include <vector>

namespace kyai {
namespace config {
namespace model {

enum class AiCapability {
    Nlp,
    Vision,
    Speech,
};

enum class DeployType {
    Ondevice,
    PublicCloud,
    PrivateCloud,
};

struct Authentication {
    std::string friendlyKeyName;
    std::string key;
    std::string value;
};

struct Model {
    std::string modelName;
    std::string vendor;
    AiCapability capability;
    DeployType deployType;
};

struct MultiAuthentication {
    std::string itemName;
    std::vector<Authentication> authentications;
};

class ModelConfigPrivate;
class ModelConfig {
public:
    ModelConfig();
    ~ModelConfig();

    std::vector<Model> getModels(AiCapability capability) const;
    std::vector<Model> getModels(DeployType deployType) const;

    std::string getCurrentModelName(AiCapability capability,
                                    DeployType deployType) const;

    std::vector<MultiAuthentication> getModelAuthentications(
        const std::string &modelName) const;

    int getModelSchedulingPriority(DeployType deployType) const;
    bool getModelDeployEnabled(DeployType deployType) const;

    bool setCurrentModelName(AiCapability capability, DeployType deployType,
                            const std::string &modelName);

    bool setModelAuthentications(
        const std::string &modelName,
        const std::vector<MultiAuthentication> authentications);

    bool setModelSchedulingPriority(DeployType deployType, int priority);

    bool setModelDeployEnabled(DeployType deployType, bool enabled);

    bool clearModelAuthentications(const std::string &modelName) const;

private:
    std::unique_ptr<ModelConfigPrivate> modelConfigPrivate_;
};

}  // namespace model
}  // namespace config
}  // namespace kyai